# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, Iterable, Optional, TypeVar
import urllib.parse

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from ..._serialization import Serializer

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_list_operations_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2023-01-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop("template_url", "/providers/Microsoft.DomainRegistration/operations")

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


class DomainRegistrationProviderOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.web.v2023_01_01.WebSiteManagementClient`'s
        :attr:`domain_registration_provider` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")
        self._api_version = input_args.pop(0) if input_args else kwargs.pop("api_version")

    @distributed_trace
    def list_operations(self, **kwargs: Any) -> Iterable["_models.CsmOperationDescription"]:
        """Implements Csm operations Api to exposes the list of available Csm Apis under the resource
        provider.

        Description for Implements Csm operations Api to exposes the list of available Csm Apis under
        the resource provider.

        :return: An iterator like instance of either CsmOperationDescription or the result of
         cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.web.v2023_01_01.models.CsmOperationDescription]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._api_version or "2023-01-01"))
        cls: ClsType[_models.CsmOperationCollection] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_operations_request(
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("CsmOperationCollection", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.DefaultErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from typing import Any, Optional, Union, cast

from azure.core.credentials import TokenCredential, AccessToken

from ._exchange_client import ExchangeClientAuthenticationPolicy
from ._generated import ContainerRegistry
from ._generated.models import TokenGrantType
from ._generated.operations._patch import AuthenticationOperations
from ._helpers import _parse_challenge
from ._user_agent import USER_AGENT


class AnonymousAccessCredential(TokenCredential):
    def get_token(
        self, *scopes: str, claims: Optional[str] = None, tenant_id: Optional[str] = None, **kwargs: Any
    ) -> AccessToken:
        raise ValueError("This credential cannot be used to obtain access tokens.")


class AnonymousACRExchangeClient(object):
    """Class for handling oauth authentication requests

    :param endpoint: Azure Container Registry endpoint
    :type endpoint: str
    :keyword api_version: API Version. The default value is "2021-07-01".
    :paramtype api_version: str
    """

    def __init__(  # pylint: disable=missing-client-constructor-parameter-credential
        self, endpoint: str, **kwargs: Any
    ) -> None:
        if not endpoint.startswith("https://") and not endpoint.startswith("http://"):
            endpoint = "https://" + endpoint
        self._endpoint = endpoint
        self._client = ContainerRegistry(
            credential=AnonymousAccessCredential(),
            url=endpoint,
            sdk_moniker=USER_AGENT,
            authentication_policy=ExchangeClientAuthenticationPolicy(),
            **kwargs
        )

    def get_acr_access_token(  # pylint:disable=client-method-missing-tracing-decorator
        self, challenge: str, **kwargs: Any
    ) -> Optional[str]:
        parsed_challenge = _parse_challenge(challenge)
        return self.exchange_refresh_token_for_access_token(
            "",
            service=parsed_challenge["service"],
            scope=parsed_challenge["scope"],
            grant_type=TokenGrantType.PASSWORD,
            **kwargs
        )

    def exchange_refresh_token_for_access_token(  # pylint:disable=client-method-missing-tracing-decorator
        self, refresh_token: str, service: str, scope: str, grant_type: Union[str, TokenGrantType], **kwargs: Any
    ) -> Optional[str]:
        auth_operation = cast(AuthenticationOperations, self._client.authentication)
        access_token = auth_operation.exchange_acr_refresh_token_for_acr_access_token(
            service=service, scope=scope, refresh_token=refresh_token, grant_type=grant_type, **kwargs
        )
        return access_token.access_token

    def __enter__(self):
        self._client.__enter__()
        return self

    def __exit__(self, *args):
        self._client.__exit__(*args)

    def close(self) -> None:
        """Close sockets opened by the client.
        Calling this method is unnecessary when using the client as a context manager.
        """
        self._client.close()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_analyze_orchestration_app_qna_response.py

DESCRIPTION:
    This sample demonstrates how to analyze user query using an orchestration project.
    In this sample, orchestration project's top intent will map to a Qna project.

    For more info about how to setup a CLU orchestration project, see the README.

USAGE:
    python sample_analyze_orchestration_app_qna_response.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_CONVERSATIONS_ENDPOINT                       - endpoint for your CLU resource.
    2) AZURE_CONVERSATIONS_KEY                            - API key for your CLU resource.
    3) AZURE_CONVERSATIONS_WORKFLOW_PROJECT_NAME     - project name for your CLU orchestration project.
    4) AZURE_CONVERSATIONS_WORKFLOW_DEPLOYMENT_NAME  - deployment name for your CLU orchestration project.
"""

def sample_analyze_orchestration_app_qna_response():
    # [START analyze_orchestration_app_qna_response]
    # import libraries
    import os
    from azure.core.credentials import AzureKeyCredential
    from azure.ai.language.conversations import ConversationAnalysisClient

    # get secrets
    clu_endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]
    clu_key = os.environ["AZURE_CONVERSATIONS_KEY"]
    project_name = os.environ["AZURE_CONVERSATIONS_WORKFLOW_PROJECT_NAME"]
    deployment_name = os.environ["AZURE_CONVERSATIONS_WORKFLOW_DEPLOYMENT_NAME"]

    # analyze query
    client = ConversationAnalysisClient(clu_endpoint, AzureKeyCredential(clu_key))
    with client:
        query = "How are you?"
        result = client.analyze_conversation(
            task={
                "kind": "Conversation",
                "analysisInput": {
                    "conversationItem": {
                        "participantId": "1",
                        "id": "1",
                        "modality": "text",
                        "language": "en",
                        "text": query
                    },
                    "isLoggingEnabled": False
                },
                "parameters": {
                    "projectName": project_name,
                    "deploymentName": deployment_name,
                    "verbose": True
                }
            }
        )

    # view result
    print(f"query: {result['result']['query']}")
    print(f"project kind: {result['result']['prediction']['projectKind']}\n")

    # top intent
    top_intent = result["result"]["prediction"]["topIntent"]
    print(f"top intent: {top_intent}")
    top_intent_object = result["result"]["prediction"]["intents"][top_intent]
    print(f"confidence score: {top_intent_object['confidenceScore']}")
    print(f"project kind: {top_intent_object['targetProjectKind']}")

    if top_intent_object["targetProjectKind"] == "QuestionAnswering":
        print("\nview qna result:")
        qna_result = top_intent_object["result"]
        for answer in qna_result["answers"]:
            print(f"\nanswer: {answer['answer']}")
            print(f"answer: {answer['confidenceScore']}")

    # [END analyze_orchestration_app_qna_response]

if __name__ == '__main__':
    sample_analyze_orchestration_app_qna_response()


# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
import os
from typing import List, Any, Optional
from typing_extensions import Self
from azure.core.credentials import TokenCredential
from azure.ai.agents import AgentsClient
from ._client import AIProjectClient as AIProjectClientGenerated
from .operations import TelemetryOperations, InferenceOperations
from ._patch_prompts import PromptTemplate
from ._patch_telemetry import enable_telemetry

_console_logging_enabled: bool = os.environ.get("ENABLE_AZURE_AI_PROJECTS_CONSOLE_LOGGING", "False").lower() in (
    "true",
    "1",
    "yes",
)
if _console_logging_enabled:
    import sys
    import logging

    azure_logger = logging.getLogger("azure")
    azure_logger.setLevel(logging.DEBUG)
    azure_logger.addHandler(logging.StreamHandler(stream=sys.stdout))
    identity_logger = logging.getLogger("azure.identity")
    identity_logger.setLevel(logging.ERROR)


def _patch_user_agent(user_agent: Optional[str]) -> str:
    # All authenticated external clients exposed by this client will have this application id
    # set on their user-agent. For more info on user-agent HTTP header, see:
    # https://azure.github.io/azure-sdk/general_azurecore.html#telemetry-policy
    USER_AGENT_APP_ID = "AIProjectClient"

    if user_agent:
        # If the calling application has set "user_agent" when constructing the AIProjectClient,
        # take that value and prepend it to USER_AGENT_APP_ID.
        patched_user_agent = f"{user_agent}-{USER_AGENT_APP_ID}"
    else:
        patched_user_agent = USER_AGENT_APP_ID

    return patched_user_agent


class AIProjectClient(AIProjectClientGenerated):  # pylint: disable=too-many-instance-attributes
    """AIProjectClient.

    :ivar agents: The AgentsClient associated with this AIProjectClient.
    :vartype agents: azure.ai.agents.AgentsClient
    :ivar connections: ConnectionsOperations operations
    :vartype connections: azure.ai.projects.operations.ConnectionsOperations
    :ivar inference: InferenceOperations operations
    :vartype inference: azure.ai.projects.operations.InferenceOperations
    :ivar telemetry: TelemetryOperations operations
    :vartype telemetry: azure.ai.projects.operations.TelemetryOperations
    :ivar evaluations: EvaluationsOperations operations
    :vartype evaluations: azure.ai.projects.operations.EvaluationsOperations
    :ivar datasets: DatasetsOperations operations
    :vartype datasets: azure.ai.projects.operations.DatasetsOperations
    :ivar indexes: IndexesOperations operations
    :vartype indexes: azure.ai.projects.operations.IndexesOperations
    :ivar deployments: DeploymentsOperations operations
    :vartype deployments: azure.ai.projects.operations.DeploymentsOperations
    :ivar red_teams: RedTeamsOperations operations
    :vartype red_teams: azure.ai.projects.operations.RedTeamsOperations
    :param endpoint: Project endpoint. In the form
     "https://<your-ai-services-account-name>.services.ai.azure.com/api/projects/_project"
     if your Foundry Hub has only one Project, or to use the default Project in your Hub. Or in the
     form "https://<your-ai-services-account-name>.services.ai.azure.com/api/projects/<your-project-name>"
     if you want to explicitly specify the Foundry Project name. Required.
    :type endpoint: str
    :param credential: Credential used to authenticate requests to the service. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :keyword api_version: The API version to use for this operation. Default value is
     "2025-05-15-preview". Note that overriding this default value may result in unsupported
     behavior.
    :paramtype api_version: str
    """

    def __init__(self, endpoint: str, credential: TokenCredential, **kwargs: Any) -> None:

        kwargs.setdefault("logging_enable", _console_logging_enabled)

        self._kwargs = kwargs.copy()
        self._patched_user_agent = _patch_user_agent(self._kwargs.pop("user_agent", None))

        super().__init__(endpoint=endpoint, credential=credential, **kwargs)

        self.telemetry = TelemetryOperations(self)  # type: ignore
        self.inference = InferenceOperations(self)  # type: ignore
        self._agents: Optional[AgentsClient] = None

    @property
    def agents(self) -> AgentsClient:  # type: ignore[name-defined]
        """Get the AgentsClient associated with this AIProjectClient.
        The package azure.ai.agents must be installed to use this property.

        :return: The AgentsClient associated with this AIProjectClient.
        :rtype: azure.ai.agents.AgentsClient
        """
        if self._agents is None:
            self._agents = AgentsClient(
                endpoint=self._config.endpoint,
                credential=self._config.credential,
                user_agent=self._patched_user_agent,
                **self._kwargs,
            )
        return self._agents

    def close(self) -> None:
        if self._agents:
            self.agents.close()
        super().close()

    def __enter__(self) -> Self:
        super().__enter__()
        if self._agents:
            self.agents.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        if self._agents:
            self.agents.__exit__(*exc_details)
        super().__exit__(*exc_details)


__all__: List[str] = [
    "AIProjectClient",
    "PromptTemplate",
    "enable_telemetry",
]  # Add all objects you want publicly available to users at this package level


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    ResourceSku,
    ResourceSkuCapabilities,
    ResourceSkuCapacity,
    ResourceSkuCosts,
    ResourceSkuLocationInfo,
    ResourceSkuRestrictionInfo,
    ResourceSkuRestrictions,
    ResourceSkusResult,
)

from ._compute_management_client_enums import (  # type: ignore
    ResourceSkuCapacityScaleType,
    ResourceSkuRestrictionsReasonCode,
    ResourceSkuRestrictionsType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ResourceSku",
    "ResourceSkuCapabilities",
    "ResourceSkuCapacity",
    "ResourceSkuCosts",
    "ResourceSkuLocationInfo",
    "ResourceSkuRestrictionInfo",
    "ResourceSkuRestrictions",
    "ResourceSkusResult",
    "ResourceSkuCapacityScaleType",
    "ResourceSkuRestrictionsReasonCode",
    "ResourceSkuRestrictionsType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import pytest

from azure.ai.language.conversations import ConversationAnalysisClient
from azure.core.credentials import AzureKeyCredential
from devtools_testutils import AzureRecordedTestCase


class TestConversationApp(AzureRecordedTestCase):

    def test_conversation_app(self, recorded_test, conversation_creds):

        # analyze query
        client = ConversationAnalysisClient(
            conversation_creds["endpoint"],
            AzureKeyCredential(conversation_creds["key"])
        )
        with client:
            query = "Send an email to Carol about the tomorrow's demo"
            result = client.analyze_conversation(
                task={
                    "kind": "Conversation",
                    "analysisInput": {
                        "conversationItem": {
                            "participantId": "1",
                            "id": "1",
                            "modality": "text",
                            "language": "en",
                            "text": query
                        },
                        "isLoggingEnabled": False
                    },
                    "parameters": {
                        "projectName": conversation_creds["conv_project_name"],
                        "deploymentName": conversation_creds["conv_deployment_name"],
                        "verbose": True
                    }
                }
            )

            # assert - main object
            assert not result is None
            assert result["kind"] == "ConversationResult"

            # assert - prediction type
            assert result["result"]["query"] == query
            assert result["result"]["prediction"]["projectKind"] == 'Conversation'

            # assert - top intent
            assert result["result"]["prediction"]["topIntent"] == 'Send'
            assert len(result["result"]["prediction"]["intents"]) > 0
            assert result["result"]["prediction"]["intents"][0]["category"] == 'Send'
            assert result["result"]["prediction"]["intents"][0]["confidenceScore"] > 0

            # assert - entities
            assert len(result["result"]["prediction"]["entities"]) > 0
            assert result["result"]["prediction"]["entities"][0]["category"] == 'Contact'
            assert result["result"]["prediction"]["entities"][0]["text"] == 'Carol'
            assert result["result"]["prediction"]["entities"][0]["confidenceScore"] > 0

    @pytest.mark.skip("no runtime AAD yet for 2023-04-01")
    def test_conversation_app_aad_auth(self, recorded_test, conversation_creds):
        token = self.get_credential(ConversationAnalysisClient)
        client = ConversationAnalysisClient(conversation_creds["endpoint"], token)
        with client:
            query = "Send an email to Carol about the tomorrow's demo"
            result = client.analyze_conversation(
                task={
                    "kind": "Conversation",
                    "analysisInput": {
                        "conversationItem": {
                            "participantId": "1",
                            "id": "1",
                            "modality": "text",
                            "language": "en",
                            "text": query
                        },
                        "isLoggingEnabled": False
                    },
                    "parameters": {
                        "projectName": conversation_creds["conv_project_name"],
                        "deploymentName": conversation_creds["conv_deployment_name"],
                        "verbose": True
                    }
                }
            )

            # assert - main object
            assert not result is None
            assert result["kind"] == "ConversationResult"

            # assert - prediction type
            assert result["result"]["query"] == query
            assert result["result"]["prediction"]["projectKind"] == 'Conversation'

            # assert - top intent
            assert result["result"]["prediction"]["topIntent"] == 'Send'
            assert len(result["result"]["prediction"]["intents"]) > 0
            assert result["result"]["prediction"]["intents"][0]["category"] == 'Send'
            assert result["result"]["prediction"]["intents"][0]["confidenceScore"] > 0

            # assert - entities
            assert len(result["result"]["prediction"]["entities"]) > 0
            assert result["result"]["prediction"]["entities"][0]["category"] == 'Contact'
            assert result["result"]["prediction"]["entities"][0]["text"] == 'Carol'
            assert result["result"]["prediction"]["entities"][0]["confidenceScore"] > 0



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._backup_vaults_operations import BackupVaultsOperations
from ._operation_result_operations import OperationResultOperations
from ._operation_status_operations import OperationStatusOperations
from ._operation_status_backup_vault_context_operations import OperationStatusBackupVaultContextOperations
from ._operation_status_resource_group_context_operations import OperationStatusResourceGroupContextOperations
from ._backup_vault_operation_results_operations import BackupVaultOperationResultsOperations
from ._data_protection_operations import DataProtectionOperations
from ._data_protection_operations_operations import DataProtectionOperationsOperations
from ._backup_policies_operations import BackupPoliciesOperations
from ._backup_instances_operations import BackupInstancesOperations
from ._recovery_points_operations import RecoveryPointsOperations
from ._fetch_secondary_recovery_points_operations import FetchSecondaryRecoveryPointsOperations
from ._fetch_cross_region_restore_job_operations import FetchCrossRegionRestoreJobOperations
from ._fetch_cross_region_restore_jobs_operations import FetchCrossRegionRestoreJobsOperations
from ._backup_instances_extension_routing_operations import BackupInstancesExtensionRoutingOperations
from ._jobs_operations import JobsOperations
from ._restorable_time_ranges_operations import RestorableTimeRangesOperations
from ._export_jobs_operations import ExportJobsOperations
from ._export_jobs_operation_result_operations import ExportJobsOperationResultOperations
from ._deleted_backup_instances_operations import DeletedBackupInstancesOperations
from ._resource_guards_operations import ResourceGuardsOperations
from ._dpp_resource_guard_proxy_operations import DppResourceGuardProxyOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "BackupVaultsOperations",
    "OperationResultOperations",
    "OperationStatusOperations",
    "OperationStatusBackupVaultContextOperations",
    "OperationStatusResourceGroupContextOperations",
    "BackupVaultOperationResultsOperations",
    "DataProtectionOperations",
    "DataProtectionOperationsOperations",
    "BackupPoliciesOperations",
    "BackupInstancesOperations",
    "RecoveryPointsOperations",
    "FetchSecondaryRecoveryPointsOperations",
    "FetchCrossRegionRestoreJobOperations",
    "FetchCrossRegionRestoreJobsOperations",
    "BackupInstancesExtensionRoutingOperations",
    "JobsOperations",
    "RestorableTimeRangesOperations",
    "ExportJobsOperations",
    "ExportJobsOperationResultOperations",
    "DeletedBackupInstancesOperations",
    "ResourceGuardsOperations",
    "DppResourceGuardProxyOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, IO, Iterable, Optional, TypeVar, Union, overload

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._serialization import Serializer
from .._vendor import _convert_request, _format_url_section

if sys.version_info >= (3, 8):
    from typing import Literal  # pylint: disable=no-name-in-module, ungrouped-imports
else:
    from typing_extensions import Literal  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_meta_data_request(*, identifier: Union[str, _models.Identifier], **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: Literal["2019-05-05-preview"] = kwargs.pop(
        "api_version", _params.pop("api-version", "2019-05-05-preview")
    )
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop("template_url", "/providers/Microsoft.AlertsManagement/alertsMetaData")

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    _params["identifier"] = _SERIALIZER.query("identifier", identifier, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_all_request(
    subscription_id: str,
    *,
    target_resource: Optional[str] = None,
    target_resource_type: Optional[str] = None,
    target_resource_group: Optional[str] = None,
    monitor_service: Optional[Union[str, _models.MonitorService]] = None,
    monitor_condition: Optional[Union[str, _models.MonitorCondition]] = None,
    severity: Optional[Union[str, _models.Severity]] = None,
    alert_state: Optional[Union[str, _models.AlertState]] = None,
    alert_rule: Optional[str] = None,
    smart_group_id: Optional[str] = None,
    include_context: Optional[bool] = None,
    include_egress_config: Optional[bool] = None,
    page_count: Optional[int] = None,
    sort_by: Optional[Union[str, _models.AlertsSortByFields]] = None,
    sort_order: Optional[Union[str, _models.SortOrder]] = None,
    select: Optional[str] = None,
    time_range: Optional[Union[str, _models.TimeRange]] = None,
    custom_time_range: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: Literal["2019-05-05-preview"] = kwargs.pop(
        "api_version", _params.pop("api-version", "2019-05-05-preview")
    )
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alerts")
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
    }

    _url: str = _format_url_section(_url, **path_format_arguments)  # type: ignore

    # Construct parameters
    if target_resource is not None:
        _params["targetResource"] = _SERIALIZER.query("target_resource", target_resource, "str")
    if target_resource_type is not None:
        _params["targetResourceType"] = _SERIALIZER.query("target_resource_type", target_resource_type, "str")
    if target_resource_group is not None:
        _params["targetResourceGroup"] = _SERIALIZER.query("target_resource_group", target_resource_group, "str")
    if monitor_service is not None:
        _params["monitorService"] = _SERIALIZER.query("monitor_service", monitor_service, "str")
    if monitor_condition is not None:
        _params["monitorCondition"] = _SERIALIZER.query("monitor_condition", monitor_condition, "str")
    if severity is not None:
        _params["severity"] = _SERIALIZER.query("severity", severity, "str")
    if alert_state is not None:
        _params["alertState"] = _SERIALIZER.query("alert_state", alert_state, "str")
    if alert_rule is not None:
        _params["alertRule"] = _SERIALIZER.query("alert_rule", alert_rule, "str")
    if smart_group_id is not None:
        _params["smartGroupId"] = _SERIALIZER.query("smart_group_id", smart_group_id, "str")
    if include_context is not None:
        _params["includeContext"] = _SERIALIZER.query("include_context", include_context, "bool")
    if include_egress_config is not None:
        _params["includeEgressConfig"] = _SERIALIZER.query("include_egress_config", include_egress_config, "bool")
    if page_count is not None:
        _params["pageCount"] = _SERIALIZER.query("page_count", page_count, "int")
    if sort_by is not None:
        _params["sortBy"] = _SERIALIZER.query("sort_by", sort_by, "str")
    if sort_order is not None:
        _params["sortOrder"] = _SERIALIZER.query("sort_order", sort_order, "str")
    if select is not None:
        _params["select"] = _SERIALIZER.query("select", select, "str")
    if time_range is not None:
        _params["timeRange"] = _SERIALIZER.query("time_range", time_range, "str")
    if custom_time_range is not None:
        _params["customTimeRange"] = _SERIALIZER.query("custom_time_range", custom_time_range, "str")
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_by_id_request(alert_id: str, subscription_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: Literal["2019-05-05-preview"] = kwargs.pop(
        "api_version", _params.pop("api-version", "2019-05-05-preview")
    )
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url", "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alerts/{alertId}"
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "alertId": _SERIALIZER.url("alert_id", alert_id, "str"),
    }

    _url: str = _format_url_section(_url, **path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_change_state_request(
    alert_id: str, subscription_id: str, *, new_state: Union[str, _models.AlertState], **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: Literal["2019-05-05-preview"] = kwargs.pop(
        "api_version", _params.pop("api-version", "2019-05-05-preview")
    )
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alerts/{alertId}/changestate",
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "alertId": _SERIALIZER.url("alert_id", alert_id, "str"),
    }

    _url: str = _format_url_section(_url, **path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    _params["newState"] = _SERIALIZER.query("new_state", new_state, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_history_request(alert_id: str, subscription_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: Literal["2019-05-05-preview"] = kwargs.pop(
        "api_version", _params.pop("api-version", "2019-05-05-preview")
    )
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url", "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alerts/{alertId}/history"
    )  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "alertId": _SERIALIZER.url("alert_id", alert_id, "str"),
    }

    _url: str = _format_url_section(_url, **path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_summary_request(
    subscription_id: str,
    *,
    groupby: Union[str, _models.AlertsSummaryGroupByFields],
    include_smart_groups_count: Optional[bool] = None,
    target_resource: Optional[str] = None,
    target_resource_type: Optional[str] = None,
    target_resource_group: Optional[str] = None,
    monitor_service: Optional[Union[str, _models.MonitorService]] = None,
    monitor_condition: Optional[Union[str, _models.MonitorCondition]] = None,
    severity: Optional[Union[str, _models.Severity]] = None,
    alert_state: Optional[Union[str, _models.AlertState]] = None,
    alert_rule: Optional[str] = None,
    time_range: Optional[Union[str, _models.TimeRange]] = None,
    custom_time_range: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: Literal["2019-05-05-preview"] = kwargs.pop(
        "api_version", _params.pop("api-version", "2019-05-05-preview")
    )
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url", "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alertsSummary"
    )
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
    }

    _url: str = _format_url_section(_url, **path_format_arguments)  # type: ignore

    # Construct parameters
    _params["groupby"] = _SERIALIZER.query("groupby", groupby, "str")
    if include_smart_groups_count is not None:
        _params["includeSmartGroupsCount"] = _SERIALIZER.query(
            "include_smart_groups_count", include_smart_groups_count, "bool"
        )
    if target_resource is not None:
        _params["targetResource"] = _SERIALIZER.query("target_resource", target_resource, "str")
    if target_resource_type is not None:
        _params["targetResourceType"] = _SERIALIZER.query("target_resource_type", target_resource_type, "str")
    if target_resource_group is not None:
        _params["targetResourceGroup"] = _SERIALIZER.query("target_resource_group", target_resource_group, "str")
    if monitor_service is not None:
        _params["monitorService"] = _SERIALIZER.query("monitor_service", monitor_service, "str")
    if monitor_condition is not None:
        _params["monitorCondition"] = _SERIALIZER.query("monitor_condition", monitor_condition, "str")
    if severity is not None:
        _params["severity"] = _SERIALIZER.query("severity", severity, "str")
    if alert_state is not None:
        _params["alertState"] = _SERIALIZER.query("alert_state", alert_state, "str")
    if alert_rule is not None:
        _params["alertRule"] = _SERIALIZER.query("alert_rule", alert_rule, "str")
    if time_range is not None:
        _params["timeRange"] = _SERIALIZER.query("time_range", time_range, "str")
    if custom_time_range is not None:
        _params["customTimeRange"] = _SERIALIZER.query("custom_time_range", custom_time_range, "str")
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


class AlertsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.alertsmanagement.AlertsManagementClient`'s
        :attr:`alerts` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def meta_data(self, identifier: Union[str, _models.Identifier], **kwargs: Any) -> _models.AlertsMetaData:
        """List alerts meta data information based on value of identifier parameter.

        :param identifier: Identification of the information to be retrieved by API call.
         "MonitorServiceList" Required.
        :type identifier: str or ~azure.mgmt.alertsmanagement.models.Identifier
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: AlertsMetaData or the result of cls(response)
        :rtype: ~azure.mgmt.alertsmanagement.models.AlertsMetaData
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2019-05-05-preview"] = kwargs.pop(
            "api_version", _params.pop("api-version", "2019-05-05-preview")
        )
        cls: ClsType[_models.AlertsMetaData] = kwargs.pop("cls", None)

        request = build_meta_data_request(
            identifier=identifier,
            api_version=api_version,
            template_url=self.meta_data.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseAutoGenerated2, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("AlertsMetaData", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    meta_data.metadata = {"url": "/providers/Microsoft.AlertsManagement/alertsMetaData"}

    @distributed_trace
    def get_all(
        self,
        target_resource: Optional[str] = None,
        target_resource_type: Optional[str] = None,
        target_resource_group: Optional[str] = None,
        monitor_service: Optional[Union[str, _models.MonitorService]] = None,
        monitor_condition: Optional[Union[str, _models.MonitorCondition]] = None,
        severity: Optional[Union[str, _models.Severity]] = None,
        alert_state: Optional[Union[str, _models.AlertState]] = None,
        alert_rule: Optional[str] = None,
        smart_group_id: Optional[str] = None,
        include_context: Optional[bool] = None,
        include_egress_config: Optional[bool] = None,
        page_count: Optional[int] = None,
        sort_by: Optional[Union[str, _models.AlertsSortByFields]] = None,
        sort_order: Optional[Union[str, _models.SortOrder]] = None,
        select: Optional[str] = None,
        time_range: Optional[Union[str, _models.TimeRange]] = None,
        custom_time_range: Optional[str] = None,
        **kwargs: Any
    ) -> Iterable["_models.Alert"]:
        """List all existing alerts, where the results can be filtered on the basis of multiple parameters
        (e.g. time range). The results can then be sorted on the basis specific fields, with the
        default being lastModifiedDateTime.

        :param target_resource: Filter by target resource( which is full ARM ID) Default value is
         select all. Default value is None.
        :type target_resource: str
        :param target_resource_type: Filter by target resource type. Default value is select all.
         Default value is None.
        :type target_resource_type: str
        :param target_resource_group: Filter by target resource group name. Default value is select
         all. Default value is None.
        :type target_resource_group: str
        :param monitor_service: Filter by monitor service which generates the alert instance. Default
         value is select all. Known values are: "Application Insights", "ActivityLog Administrative",
         "ActivityLog Security", "ActivityLog Recommendation", "ActivityLog Policy", "ActivityLog
         Autoscale", "Log Analytics", "Nagios", "Platform", "SCOM", "ServiceHealth", "SmartDetector",
         "VM Insights", and "Zabbix". Default value is None.
        :type monitor_service: str or ~azure.mgmt.alertsmanagement.models.MonitorService
        :param monitor_condition: Filter by monitor condition which is either 'Fired' or 'Resolved'.
         Default value is to select all. Known values are: "Fired" and "Resolved". Default value is
         None.
        :type monitor_condition: str or ~azure.mgmt.alertsmanagement.models.MonitorCondition
        :param severity: Filter by severity.  Default value is select all. Known values are: "Sev0",
         "Sev1", "Sev2", "Sev3", and "Sev4". Default value is None.
        :type severity: str or ~azure.mgmt.alertsmanagement.models.Severity
        :param alert_state: Filter by state of the alert instance. Default value is to select all.
         Known values are: "New", "Acknowledged", and "Closed". Default value is None.
        :type alert_state: str or ~azure.mgmt.alertsmanagement.models.AlertState
        :param alert_rule: Filter by specific alert rule.  Default value is to select all. Default
         value is None.
        :type alert_rule: str
        :param smart_group_id: Filter the alerts list by the Smart Group Id. Default value is none.
         Default value is None.
        :type smart_group_id: str
        :param include_context: Include context which has contextual data specific to the monitor
         service. Default value is false'. Default value is None.
        :type include_context: bool
        :param include_egress_config: Include egress config which would be used for displaying the
         content in portal.  Default value is 'false'. Default value is None.
        :type include_egress_config: bool
        :param page_count: Determines number of alerts returned per page in response. Permissible value
         is between 1 to 250. When the "includeContent"  filter is selected, maximum value allowed is
         25. Default value is 25. Default value is None.
        :type page_count: int
        :param sort_by: Sort the query results by input field,  Default value is
         'lastModifiedDateTime'. Known values are: "name", "severity", "alertState", "monitorCondition",
         "targetResource", "targetResourceName", "targetResourceGroup", "targetResourceType",
         "startDateTime", and "lastModifiedDateTime". Default value is None.
        :type sort_by: str or ~azure.mgmt.alertsmanagement.models.AlertsSortByFields
        :param sort_order: Sort the query results order in either ascending or descending.  Default
         value is 'desc' for time fields and 'asc' for others. Known values are: "asc" and "desc".
         Default value is None.
        :type sort_order: str or ~azure.mgmt.alertsmanagement.models.SortOrder
        :param select: This filter allows to selection of the fields(comma separated) which would  be
         part of the essential section. This would allow to project only the  required fields rather
         than getting entire content.  Default is to fetch all the fields in the essentials section.
         Default value is None.
        :type select: str
        :param time_range: Filter by time range by below listed values. Default value is 1 day. Known
         values are: "1h", "1d", "7d", and "30d". Default value is None.
        :type time_range: str or ~azure.mgmt.alertsmanagement.models.TimeRange
        :param custom_time_range: Filter by custom time range in the format
         :code:`<start-time>`/:code:`<end-time>`  where time is in (ISO-8601 format)'. Permissible
         values is within 30 days from  query time. Either timeRange or customTimeRange could be used
         but not both. Default is none. Default value is None.
        :type custom_time_range: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either Alert or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.alertsmanagement.models.Alert]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2019-05-05-preview"] = kwargs.pop(
            "api_version", _params.pop("api-version", "2019-05-05-preview")
        )
        cls: ClsType[_models.AlertsList] = kwargs.pop("cls", None)

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_get_all_request(
                    subscription_id=self._config.subscription_id,
                    target_resource=target_resource,
                    target_resource_type=target_resource_type,
                    target_resource_group=target_resource_group,
                    monitor_service=monitor_service,
                    monitor_condition=monitor_condition,
                    severity=severity,
                    alert_state=alert_state,
                    alert_rule=alert_rule,
                    smart_group_id=smart_group_id,
                    include_context=include_context,
                    include_egress_config=include_egress_config,
                    page_count=page_count,
                    sort_by=sort_by,
                    sort_order=sort_order,
                    select=select,
                    time_range=time_range,
                    custom_time_range=custom_time_range,
                    api_version=api_version,
                    template_url=self.get_all.metadata["url"],
                    headers=_headers,
                    params=_params,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                request = HttpRequest("GET", next_link)
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("AlertsList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                request, stream=False, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponseAutoGenerated2, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    get_all.metadata = {"url": "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alerts"}

    @distributed_trace
    def get_by_id(self, alert_id: str, **kwargs: Any) -> _models.Alert:
        """Get a specific alert.

        Get information related to a specific alert.

        :param alert_id: Unique ID of an alert instance. Required.
        :type alert_id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Alert or the result of cls(response)
        :rtype: ~azure.mgmt.alertsmanagement.models.Alert
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2019-05-05-preview"] = kwargs.pop(
            "api_version", _params.pop("api-version", "2019-05-05-preview")
        )
        cls: ClsType[_models.Alert] = kwargs.pop("cls", None)

        request = build_get_by_id_request(
            alert_id=alert_id,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.get_by_id.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseAutoGenerated2, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Alert", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_by_id.metadata = {
        "url": "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alerts/{alertId}"
    }

    @overload
    def change_state(
        self,
        alert_id: str,
        new_state: Union[str, _models.AlertState],
        comment: Optional[_models.Comments] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.Alert:
        """Change the state of an alert.

        :param alert_id: Unique ID of an alert instance. Required.
        :type alert_id: str
        :param new_state: New state of the alert. Known values are: "New", "Acknowledged", and
         "Closed". Required.
        :type new_state: str or ~azure.mgmt.alertsmanagement.models.AlertState
        :param comment: reason of change alert state. Default value is None.
        :type comment: ~azure.mgmt.alertsmanagement.models.Comments
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Alert or the result of cls(response)
        :rtype: ~azure.mgmt.alertsmanagement.models.Alert
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def change_state(
        self,
        alert_id: str,
        new_state: Union[str, _models.AlertState],
        comment: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.Alert:
        """Change the state of an alert.

        :param alert_id: Unique ID of an alert instance. Required.
        :type alert_id: str
        :param new_state: New state of the alert. Known values are: "New", "Acknowledged", and
         "Closed". Required.
        :type new_state: str or ~azure.mgmt.alertsmanagement.models.AlertState
        :param comment: reason of change alert state. Default value is None.
        :type comment: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Alert or the result of cls(response)
        :rtype: ~azure.mgmt.alertsmanagement.models.Alert
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def change_state(
        self,
        alert_id: str,
        new_state: Union[str, _models.AlertState],
        comment: Optional[Union[_models.Comments, IO]] = None,
        **kwargs: Any
    ) -> _models.Alert:
        """Change the state of an alert.

        :param alert_id: Unique ID of an alert instance. Required.
        :type alert_id: str
        :param new_state: New state of the alert. Known values are: "New", "Acknowledged", and
         "Closed". Required.
        :type new_state: str or ~azure.mgmt.alertsmanagement.models.AlertState
        :param comment: reason of change alert state. Is either a model type or a IO type. Default
         value is None.
        :type comment: ~azure.mgmt.alertsmanagement.models.Comments or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Alert or the result of cls(response)
        :rtype: ~azure.mgmt.alertsmanagement.models.Alert
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2019-05-05-preview"] = kwargs.pop(
            "api_version", _params.pop("api-version", "2019-05-05-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.Alert] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(comment, (IO, bytes)):
            _content = comment
        else:
            if comment is not None:
                _json = self._serialize.body(comment, "Comments")
            else:
                _json = None

        request = build_change_state_request(
            alert_id=alert_id,
            subscription_id=self._config.subscription_id,
            new_state=new_state,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.change_state.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseAutoGenerated2, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Alert", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    change_state.metadata = {
        "url": "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alerts/{alertId}/changestate"
    }

    @distributed_trace
    def get_history(self, alert_id: str, **kwargs: Any) -> _models.AlertModification:
        """Get the history of an alert, which captures any monitor condition changes (Fired/Resolved) and
        alert state changes (New/Acknowledged/Closed).

        :param alert_id: Unique ID of an alert instance. Required.
        :type alert_id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: AlertModification or the result of cls(response)
        :rtype: ~azure.mgmt.alertsmanagement.models.AlertModification
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2019-05-05-preview"] = kwargs.pop(
            "api_version", _params.pop("api-version", "2019-05-05-preview")
        )
        cls: ClsType[_models.AlertModification] = kwargs.pop("cls", None)

        request = build_get_history_request(
            alert_id=alert_id,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.get_history.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseAutoGenerated2, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("AlertModification", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_history.metadata = {
        "url": "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alerts/{alertId}/history"
    }

    @distributed_trace
    def get_summary(
        self,
        groupby: Union[str, _models.AlertsSummaryGroupByFields],
        include_smart_groups_count: Optional[bool] = None,
        target_resource: Optional[str] = None,
        target_resource_type: Optional[str] = None,
        target_resource_group: Optional[str] = None,
        monitor_service: Optional[Union[str, _models.MonitorService]] = None,
        monitor_condition: Optional[Union[str, _models.MonitorCondition]] = None,
        severity: Optional[Union[str, _models.Severity]] = None,
        alert_state: Optional[Union[str, _models.AlertState]] = None,
        alert_rule: Optional[str] = None,
        time_range: Optional[Union[str, _models.TimeRange]] = None,
        custom_time_range: Optional[str] = None,
        **kwargs: Any
    ) -> _models.AlertsSummary:
        """Get a summarized count of your alerts grouped by various parameters (e.g. grouping by
        'Severity' returns the count of alerts for each severity).

        :param groupby: This parameter allows the result set to be grouped by input fields (Maximum 2
         comma separated fields supported). For example, groupby=severity or
         groupby=severity,alertstate. Known values are: "severity", "alertState", "monitorCondition",
         "monitorService", "signalType", and "alertRule". Required.
        :type groupby: str or ~azure.mgmt.alertsmanagement.models.AlertsSummaryGroupByFields
        :param include_smart_groups_count: Include count of the SmartGroups as part of the summary.
         Default value is 'false'. Default value is None.
        :type include_smart_groups_count: bool
        :param target_resource: Filter by target resource( which is full ARM ID) Default value is
         select all. Default value is None.
        :type target_resource: str
        :param target_resource_type: Filter by target resource type. Default value is select all.
         Default value is None.
        :type target_resource_type: str
        :param target_resource_group: Filter by target resource group name. Default value is select
         all. Default value is None.
        :type target_resource_group: str
        :param monitor_service: Filter by monitor service which generates the alert instance. Default
         value is select all. Known values are: "Application Insights", "ActivityLog Administrative",
         "ActivityLog Security", "ActivityLog Recommendation", "ActivityLog Policy", "ActivityLog
         Autoscale", "Log Analytics", "Nagios", "Platform", "SCOM", "ServiceHealth", "SmartDetector",
         "VM Insights", and "Zabbix". Default value is None.
        :type monitor_service: str or ~azure.mgmt.alertsmanagement.models.MonitorService
        :param monitor_condition: Filter by monitor condition which is either 'Fired' or 'Resolved'.
         Default value is to select all. Known values are: "Fired" and "Resolved". Default value is
         None.
        :type monitor_condition: str or ~azure.mgmt.alertsmanagement.models.MonitorCondition
        :param severity: Filter by severity.  Default value is select all. Known values are: "Sev0",
         "Sev1", "Sev2", "Sev3", and "Sev4". Default value is None.
        :type severity: str or ~azure.mgmt.alertsmanagement.models.Severity
        :param alert_state: Filter by state of the alert instance. Default value is to select all.
         Known values are: "New", "Acknowledged", and "Closed". Default value is None.
        :type alert_state: str or ~azure.mgmt.alertsmanagement.models.AlertState
        :param alert_rule: Filter by specific alert rule.  Default value is to select all. Default
         value is None.
        :type alert_rule: str
        :param time_range: Filter by time range by below listed values. Default value is 1 day. Known
         values are: "1h", "1d", "7d", and "30d". Default value is None.
        :type time_range: str or ~azure.mgmt.alertsmanagement.models.TimeRange
        :param custom_time_range: Filter by custom time range in the format
         :code:`<start-time>`/:code:`<end-time>`  where time is in (ISO-8601 format)'. Permissible
         values is within 30 days from  query time. Either timeRange or customTimeRange could be used
         but not both. Default is none. Default value is None.
        :type custom_time_range: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: AlertsSummary or the result of cls(response)
        :rtype: ~azure.mgmt.alertsmanagement.models.AlertsSummary
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2019-05-05-preview"] = kwargs.pop(
            "api_version", _params.pop("api-version", "2019-05-05-preview")
        )
        cls: ClsType[_models.AlertsSummary] = kwargs.pop("cls", None)

        request = build_get_summary_request(
            subscription_id=self._config.subscription_id,
            groupby=groupby,
            include_smart_groups_count=include_smart_groups_count,
            target_resource=target_resource,
            target_resource_type=target_resource_type,
            target_resource_group=target_resource_group,
            monitor_service=monitor_service,
            monitor_condition=monitor_condition,
            severity=severity,
            alert_state=alert_state,
            alert_rule=alert_rule,
            time_range=time_range,
            custom_time_range=custom_time_range,
            api_version=api_version,
            template_url=self.get_summary.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseAutoGenerated2, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("AlertsSummary", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_summary.metadata = {"url": "/subscriptions/{subscriptionId}/providers/Microsoft.AlertsManagement/alertsSummary"}



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------

# NOTE: These tests are heavily inspired from the httpx test suite: https://github.com/encode/httpx/tree/master/tests
# Thank you httpx for your wonderful tests!
import io
import pytest
import os

try:
    import collections.abc as collections
except ImportError:
    import collections  # type: ignore

from corehttp.rest import HttpRequest
from corehttp.runtime.policies import SansIOHTTPPolicy
from rest_client import MockRestClient
from utils import NamedIo


@pytest.fixture
def assert_iterator_body():
    def _comparer(request, final_value):
        content = b"".join([p for p in request.content])
        assert content == final_value

    return _comparer


def test_request_repr():
    request = HttpRequest("GET", "http://example.org")
    assert repr(request) == "<HttpRequest [GET], url: 'http://example.org'>"


def test_no_content():
    request = HttpRequest("GET", "http://example.org")
    assert "Content-Length" not in request.headers


def test_content_length_header():
    request = HttpRequest("POST", "http://example.org", content=b"test 123")
    assert request.headers["Content-Length"] == "8"


def test_iterable_content(assert_iterator_body):
    class Content:
        def __iter__(self):
            yield b"test 123"  # pragma: nocover

    request = HttpRequest("POST", "http://example.org", content=Content())
    assert request.headers == {}
    assert_iterator_body(request, b"test 123")


def test_generator_with_transfer_encoding_header(assert_iterator_body):
    def content():
        yield b"test 123"  # pragma: nocover

    request = HttpRequest("POST", "http://example.org", content=content())
    assert request.headers == {}
    assert_iterator_body(request, b"test 123")


def test_generator_with_content_length_header(assert_iterator_body):
    def content():
        yield b"test 123"  # pragma: nocover

    headers = {"Content-Length": "8"}
    request = HttpRequest("POST", "http://example.org", content=content(), headers=headers)
    assert request.headers == {"Content-Length": "8"}
    assert_iterator_body(request, b"test 123")


def test_url_encoded_data():
    request = HttpRequest("POST", "http://example.org", data={"test": "123"})

    assert request.headers["Content-Type"] == "application/x-www-form-urlencoded"
    assert request.content == {
        "test": "123"
    }  # httpx makes this just b'test=123'. set_formdata_body is still keeping it as a dict


def test_json_encoded_data():
    request = HttpRequest("POST", "http://example.org", json={"test": 123})

    assert request.headers["Content-Type"] == "application/json"
    assert request.content == '{"test": 123}'


def test_headers():
    request = HttpRequest("POST", "http://example.org", json={"test": 123})

    assert request.headers == {
        "Content-Type": "application/json",
        "Content-Length": "13",
    }


def test_ignore_transfer_encoding_header_if_content_length_exists():
    """
    `Transfer-Encoding` should be ignored if `Content-Length` has been set explicitly.
    See https://github.com/encode/httpx/issues/1168
    """

    def streaming_body(data):
        yield data  # pragma: nocover

    data = streaming_body(b"abcd")

    headers = {"Content-Length": "4"}
    request = HttpRequest("POST", "http://example.org", data=data, headers=headers)
    assert "Transfer-Encoding" not in request.headers
    assert request.headers["Content-Length"] == "4"


def test_override_accept_encoding_header():
    headers = {"Accept-Encoding": "identity"}

    request = HttpRequest("GET", "http://example.org", headers=headers)
    assert request.headers["Accept-Encoding"] == "identity"


"""Test request body"""


def test_empty_content():
    request = HttpRequest("GET", "http://example.org")
    assert request.content is None


def test_string_content():
    request = HttpRequest("PUT", "http://example.org", content="Hello, world!")
    assert request.headers == {"Content-Length": "13", "Content-Type": "text/plain"}
    assert request.content == "Hello, world!"

    # Support 'data' for compat with requests.
    request = HttpRequest("PUT", "http://example.org", data="Hello, world!")

    assert request.headers == {"Content-Length": "13", "Content-Type": "text/plain"}
    assert request.content == "Hello, world!"

    # content length should not be set for GET requests

    request = HttpRequest("GET", "http://example.org", data="Hello, world!")

    assert request.headers == {"Content-Length": "13", "Content-Type": "text/plain"}
    assert request.content == "Hello, world!"


def test_bytes_content():
    request = HttpRequest("PUT", "http://example.org", content=b"Hello, world!")
    assert request.headers == {"Content-Length": "13"}
    assert request.content == b"Hello, world!"

    # Support 'data' for compat with requests.
    request = HttpRequest("PUT", "http://example.org", data=b"Hello, world!")

    assert request.headers == {"Content-Length": "13"}
    assert request.content == b"Hello, world!"

    # should still be set regardless of method

    request = HttpRequest("GET", "http://example.org", data=b"Hello, world!")

    assert request.headers == {"Content-Length": "13"}
    assert request.content == b"Hello, world!"


def test_iterator_content(assert_iterator_body):
    # NOTE: in httpx, content reads out the actual value. Don't do that (yet).
    def hello_world():
        yield b"Hello, "
        yield b"world!"

    request = HttpRequest("POST", url="http://example.org", content=hello_world())
    assert isinstance(request.content, collections.Iterable)

    assert_iterator_body(request, b"Hello, world!")
    assert request.headers == {}

    # Support 'data' for compat with requests.
    request = HttpRequest("POST", url="http://example.org", data=hello_world())
    assert isinstance(request.content, collections.Iterable)

    assert_iterator_body(request, b"Hello, world!")
    assert request.headers == {}

    # transfer encoding should still be set for GET requests
    request = HttpRequest("GET", url="http://example.org", data=hello_world())
    assert isinstance(request.content, collections.Iterable)

    assert_iterator_body(request, b"Hello, world!")
    assert request.headers == {}


def test_json_content():
    request = HttpRequest("POST", url="http://example.org", json={"Hello": "world!"})

    assert request.headers == {
        "Content-Length": "19",
        "Content-Type": "application/json",
    }
    assert request.content == '{"Hello": "world!"}'


def test_urlencoded_content():
    # NOTE: not adding content length setting and content testing bc we're not adding content length in the rest code
    # that's dealt with later in the pipeline.
    request = HttpRequest("POST", url="http://example.org", data={"Hello": "world!"})
    assert request.headers == {
        "Content-Type": "application/x-www-form-urlencoded",
    }


@pytest.mark.parametrize(("key"), (1, 2.3, None))
def test_multipart_invalid_key(key):

    data = {key: "abc"}
    files = {"file": io.BytesIO(b"<file content>")}
    with pytest.raises(TypeError) as e:
        HttpRequest(
            url="http://localhost:8000/",
            method="POST",
            data=data,
            files=files,
        )
    assert "Invalid type for data name" in str(e.value)
    assert repr(key) in str(e.value)


def test_multipart_invalid_key_binary_string():

    data = {b"abc": "abc"}
    files = {"file": io.BytesIO(b"<file content>")}
    with pytest.raises(TypeError) as e:
        HttpRequest(
            url="http://localhost:8000/",
            method="POST",
            data=data,
            files=files,
        )
    assert "Invalid type for data name" in str(e.value)
    assert repr(b"abc") in str(e.value)


def test_data_str_input():
    data = {
        "scope": "fake_scope",
        "grant_type": "refresh_token",
        "refresh_token": "REDACTED",
        "service": "fake_url.foo.io",
    }
    request = HttpRequest("POST", "http://localhost:3000/", data=data)
    assert len(request.content) == 4
    assert request.content["scope"] == "fake_scope"
    assert request.content["grant_type"] == "refresh_token"
    assert request.content["refresh_token"] == "REDACTED"
    assert request.content["service"] == "fake_url.foo.io"
    assert len(request.headers) == 1
    assert request.headers["Content-Type"] == "application/x-www-form-urlencoded"


def test_content_str_input():
    requests = [
        HttpRequest("POST", "/fake", content="hello, world!"),
        HttpRequest("POST", "/fake", content="hello, world!"),
    ]
    for request in requests:
        assert len(request.headers) == 2
        assert request.headers["Content-Type"] == "text/plain"
        assert request.headers["Content-Length"] == "13"
        assert request.content == "hello, world!"


@pytest.mark.parametrize(("value"), (object(), {"key": "value"}))
def test_multipart_invalid_value(value):

    data = {"text": value}
    files = {"file": io.BytesIO(b"<file content>")}
    with pytest.raises(TypeError) as e:
        HttpRequest("POST", "http://localhost:8000/", data=data, files=files)
    assert "Invalid type for data value" in str(e.value)


def test_empty_request():
    request = HttpRequest("POST", url="http://example.org", data={}, files={})

    assert request.headers == {}
    assert not request.content  # in core, we don't convert urlencoded dict to bytes representation in content


def test_read_content(assert_iterator_body):
    def content():
        yield b"test 123"

    request = HttpRequest("POST", "http://example.org", content=content())
    assert_iterator_body(request, b"test 123")
    # in this case, request._data is what we end up passing to the requests transport
    assert isinstance(request._data, collections.Iterable)


def test_complicated_json(client):
    # thanks to Sean Kane for this test!
    input = {
        "EmptyByte": "",
        "EmptyUnicode": "",
        "SpacesOnlyByte": "   ",
        "SpacesOnlyUnicode": "   ",
        "SpacesBeforeByte": "   Text",
        "SpacesBeforeUnicode": "   Text",
        "SpacesAfterByte": "Text   ",
        "SpacesAfterUnicode": "Text   ",
        "SpacesBeforeAndAfterByte": "   Text   ",
        "SpacesBeforeAndAfterUnicode": "   Text   ",
        "啊齄丂狛": "ꀕ",
        "RowKey": "test2",
        "啊齄丂狛狜": "hello",
        "singlequote": "a''''b",
        "doublequote": 'a""""b',
        "None": None,
    }
    request = HttpRequest("POST", "/basic/complicated-json", json=input)
    r = client.send_request(request)
    r.raise_for_status()


def test_use_custom_json_encoder():
    # this is to test we're using corehttp.serialization.CoreJSONEncoder
    # to serialize our JSON objects
    # since json can't serialize bytes by default but CoreJSONEncoder can,
    # we pass in bytes and check that they are serialized
    request = HttpRequest("GET", "/headers", json=bytearray("mybytes", "utf-8"))
    assert request.content == '"bXlieXRlcw=="'  # cspell:disable-line


def test_request_policies_chain(port):
    class NewPolicyOne(SansIOHTTPPolicy):
        def on_request(self, request):
            request.http_request = HttpRequest(
                request.http_request.method,
                request.http_request.url,
                headers=request.http_request.headers,
                json={"hello": "world"},
            )

    class NewPolicyModifyHeaders(SansIOHTTPPolicy):
        def on_request(self, request):
            assert request.http_request.content == '{"hello": "world"}'

            # modify header to know we entered this callback
            request.http_request.headers = {
                "x-date": "Thu, 14 Jun 2018 16:46:54 GMT",
                "Authorization": "SharedKey account:G4jjBXA7LI/RnWKIOQ8i9xH4p76pAQ+4Fs4R1VxasaE=",  # fake key suppressed in credscan
                "Content-Length": "0",
            }

    class NewPolicyValidateRequest(SansIOHTTPPolicy):
        def on_request(self, request):
            expected = {
                "x-date": "Thu, 14 Jun 2018 16:46:54 GMT",
                "Authorization": "SharedKey account:G4jjBXA7LI/RnWKIOQ8i9xH4p76pAQ+4Fs4R1VxasaE=",  # fake key suppressed in credscan
                "Content-Length": "0",
            }
            assert request.http_request.headers == expected
            assert request.http_request.content == '{"hello": "world"}'
            raise ValueError("Passed through the policies!")

    policies = [NewPolicyOne(), NewPolicyModifyHeaders(), NewPolicyValidateRequest()]
    request = HttpRequest("DELETE", "/container0/blob0")
    client = MockRestClient(port="5000", policies=policies)
    with pytest.raises(ValueError) as ex:
        client.send_request(
            request,
            content="I should be overridden",
        )
    assert "Passed through the policies!" in str(ex.value)


def test_json_file_valid():
    json_bytes = bytearray('{"more": "cowbell"}', encoding="utf-8")
    with io.BytesIO(json_bytes) as json_file:
        request = HttpRequest("PUT", "/fake", json=json_file)
        assert request.headers == {"Content-Type": "application/json"}
        assert request.content == json_file
        assert not request.content.closed
        assert request.content.read() == b'{"more": "cowbell"}'


def test_json_file_invalid():
    json_bytes = bytearray('{"more": "cowbell" i am not valid', encoding="utf-8")
    with io.BytesIO(json_bytes) as json_file:
        request = HttpRequest("PUT", "/fake", json=json_file)
        assert request.headers == {"Content-Type": "application/json"}
        assert request.content == json_file
        assert not request.content.closed
        assert request.content.read() == b'{"more": "cowbell" i am not valid'


def test_json_file_content_type_input():
    json_bytes = bytearray('{"more": "cowbell"}', encoding="utf-8")
    with io.BytesIO(json_bytes) as json_file:
        request = HttpRequest("PUT", "/fake", json=json_file, headers={"Content-Type": "application/json-special"})
        assert request.headers == {"Content-Type": "application/json-special"}
        assert request.content == json_file
        assert not request.content.closed
        assert request.content.read() == b'{"more": "cowbell"}'


class NonSeekableStream:
    def __init__(self, wrapped_stream):
        self.wrapped_stream = wrapped_stream

    def write(self, data):
        self.wrapped_stream.write(data)

    def read(self, count):
        raise ValueError("Request should not read me!")

    def seek(self, *args, **kwargs):
        raise ValueError("Can't seek!")

    def tell(self):
        return self.wrapped_stream.tell()


def test_non_seekable_stream_input():
    data = b"a" * 4 * 1024
    data_stream = NonSeekableStream(io.BytesIO(data))
    HttpRequest(method="PUT", url="http://www.example.com", content=data_stream)  # ensure we can make this HttpRequest


class Stream:
    def __init__(self, length, initial_buffer_length=4 * 1024):
        self._base_data = os.urandom(initial_buffer_length)
        self._base_data_length = initial_buffer_length
        self._position = 0
        self._remaining = length

    def read(self, size=None):
        raise ValueError("Request should not read me!")

    def remaining(self):
        return self._remaining


def test_stream_input():
    data_stream = Stream(length=4)
    HttpRequest(method="PUT", url="http://www.example.com", content=data_stream)  # ensure we can make this HttpRequest


@pytest.fixture
def filebytes():
    file_path = os.path.join(os.path.dirname(__file__), "./conftest.py")
    return open(file_path, "rb")


def test_multipart_bytes(filebytes):
    request = HttpRequest("POST", url="http://example.org", files={"file": filebytes})

    assert request.content == {"file": ("conftest.py", filebytes, "application/octet-stream")}


def test_multipart_filename_and_bytes(filebytes):
    files = ("specifiedFileName", filebytes)
    request = HttpRequest("POST", url="http://example.org", files={"file": files})

    assert request.content == {"file": ("specifiedFileName", filebytes)}


def test_multipart_filename_and_bytes_and_content_type(filebytes):
    files = ("specifiedFileName", filebytes, "application/json")
    request = HttpRequest("POST", url="http://example.org", files={"file": files})

    assert request.content == {"file": ("specifiedFileName", filebytes, "application/json")}


def test_multipart_incorrect_tuple_entry(filebytes):
    files = ("specifiedFileName", filebytes, "application/json", "extra")
    with pytest.raises(ValueError):
        HttpRequest("POST", url="http://example.org", files={"file": files})


def test_multipart_tuple_input_single(filebytes):
    request = HttpRequest("POST", url="http://example.org", files=[("file", filebytes)])

    assert request.content == [("file", ("conftest.py", filebytes, "application/octet-stream"))]


def test_multipart_tuple_input_multiple(filebytes):
    request = HttpRequest("POST", url="http://example.org", files=[("file", filebytes), ("file2", filebytes)])

    assert request.content == [
        ("file", ("conftest.py", filebytes, "application/octet-stream")),
        ("file2", ("conftest.py", filebytes, "application/octet-stream")),
    ]


def test_multipart_tuple_input_multiple_with_filename_and_content_type(filebytes):
    request = HttpRequest(
        "POST",
        url="http://example.org",
        files=[("file", ("first file", filebytes, "image/pdf")), ("file2", ("second file", filebytes, "image/png"))],
    )

    assert request.content == [
        ("file", ("first file", filebytes, "image/pdf")),
        ("file2", ("second file", filebytes, "image/png")),
    ]


def test_multipart_tuple_input_multiple_same_name(client):
    request = HttpRequest(
        "POST",
        url="/multipart/tuple-input-multiple-same-name",
        files=[
            ("file", ("firstFileName", NamedIo("firstFile"), "image/pdf")),
            ("file", ("secondFileName", NamedIo("secondFile"), "image/png")),
        ],
    )
    client.send_request(request).raise_for_status()


def test_multipart_tuple_input_multiple_same_name_with_tuple_file_value(client):
    request = HttpRequest(
        "POST",
        url="/multipart/tuple-input-multiple-same-name-with-tuple-file-value",
        files=[("images", ("foo.png", NamedIo("notMyName.pdf"), "image/png")), ("images", NamedIo("foo.png"))],
    )
    client.send_request(request).raise_for_status()


def test_data_and_file_input_same_name(client):
    request = HttpRequest(
        "POST",
        url="/multipart/data-and-file-input-same-name",
        data={"message": "Hello, world!"},
        files=[
            ("file", ("firstFileName", NamedIo("firstFile"), "image/pdf")),
            ("file", ("secondFileName", NamedIo("secondFile"), "image/png")),
        ],
    )
    client.send_request(request).raise_for_status()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AccessControlEntryAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Action object."""

    PERMIT = "Permit"
    DENY = "Deny"


class AppServicePlanRestrictions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """App Service plans this offer is restricted to."""

    NONE = "None"
    FREE = "Free"
    SHARED = "Shared"
    BASIC = "Basic"
    STANDARD = "Standard"
    PREMIUM = "Premium"


class AutoHealActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Predefined action to be taken."""

    RECYCLE = "Recycle"
    LOG_EVENT = "LogEvent"
    CUSTOM_ACTION = "CustomAction"


class Channels(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """List of channels that this recommendation can apply."""

    NOTIFICATION = "Notification"
    API = "Api"
    EMAIL = "Email"
    WEBHOOK = "Webhook"
    ALL = "All"


class CheckNameResourceTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Resource type used for verification."""

    SITE = "Site"
    SLOT = "Slot"
    HOSTING_ENVIRONMENT = "HostingEnvironment"
    PUBLISHING_USER = "PublishingUser"
    MICROSOFT_WEB_SITES = "Microsoft.Web/sites"
    MICROSOFT_WEB_SITES_SLOTS = "Microsoft.Web/sites/slots"
    MICROSOFT_WEB_HOSTING_ENVIRONMENTS = "Microsoft.Web/hostingEnvironments"
    MICROSOFT_WEB_PUBLISHING_USERS = "Microsoft.Web/publishingUsers"


class ComputeModeOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Shared/dedicated workers."""

    SHARED = "Shared"
    DEDICATED = "Dedicated"
    DYNAMIC = "Dynamic"


class ConnectionStringType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of database."""

    MY_SQL = "MySql"
    SQL_SERVER = "SQLServer"
    SQL_AZURE = "SQLAzure"
    CUSTOM = "Custom"
    NOTIFICATION_HUB = "NotificationHub"
    SERVICE_BUS = "ServiceBus"
    EVENT_HUB = "EventHub"
    API_HUB = "ApiHub"
    DOC_DB = "DocDb"
    REDIS_CACHE = "RedisCache"
    POSTGRE_SQL = "PostgreSQL"


class Enum0(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum0."""

    WINDOWS = "Windows"
    LINUX = "Linux"


class Enum1(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum1."""

    WINDOWS = "Windows"
    LINUX = "Linux"


class HostingEnvironmentStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Current status of the App Service Environment."""

    PREPARING = "Preparing"
    READY = "Ready"
    SCALING = "Scaling"
    DELETING = "Deleting"


class HostType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Indicates whether the hostname is a standard or repository hostname."""

    STANDARD = "Standard"
    REPOSITORY = "Repository"


class InAvailabilityReasonType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """:code:`<code>Invalid</code>` indicates the name provided does not match Azure App Service
    naming requirements. :code:`<code>AlreadyExists</code>` indicates that the name is already in
    use and is therefore unavailable.
    """

    INVALID = "Invalid"
    ALREADY_EXISTS = "AlreadyExists"


class InternalLoadBalancingMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies which endpoints to serve internally in the Virtual Network for the App Service
    Environment.
    """

    NONE = "None"
    WEB = "Web"
    PUBLISHING = "Publishing"


class IssueType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Represents the type of the Detector."""

    SERVICE_INCIDENT = "ServiceIncident"
    APP_DEPLOYMENT = "AppDeployment"
    APP_CRASH = "AppCrash"
    RUNTIME_ISSUE_DETECTED = "RuntimeIssueDetected"
    ASE_DEPLOYMENT = "AseDeployment"
    USER_ISSUE = "UserIssue"
    PLATFORM_ISSUE = "PlatformIssue"
    OTHER = "Other"


class KeyVaultSecretStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the Key Vault secret."""

    INITIALIZED = "Initialized"
    WAITING_ON_CERTIFICATE_ORDER = "WaitingOnCertificateOrder"
    SUCCEEDED = "Succeeded"
    CERTIFICATE_ORDER_FAILED = "CertificateOrderFailed"
    OPERATION_NOT_PERMITTED_ON_KEY_VAULT = "OperationNotPermittedOnKeyVault"
    AZURE_SERVICE_UNAUTHORIZED_TO_ACCESS_KEY_VAULT = "AzureServiceUnauthorizedToAccessKeyVault"
    KEY_VAULT_DOES_NOT_EXIST = "KeyVaultDoesNotExist"
    KEY_VAULT_SECRET_DOES_NOT_EXIST = "KeyVaultSecretDoesNotExist"
    UNKNOWN_ERROR = "UnknownError"
    EXTERNAL_PRIVATE_KEY = "ExternalPrivateKey"
    UNKNOWN = "Unknown"


class ManagedPipelineMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Managed pipeline mode."""

    INTEGRATED = "Integrated"
    CLASSIC = "Classic"


class ManagedServiceIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of managed service identity."""

    SYSTEM_ASSIGNED = "SystemAssigned"


class NotificationLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Level indicating how critical this recommendation can impact."""

    CRITICAL = "Critical"
    WARNING = "Warning"
    INFORMATION = "Information"
    NON_URGENT_SUGGESTION = "NonUrgentSuggestion"


class OperationStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The current status of the operation."""

    IN_PROGRESS = "InProgress"
    FAILED = "Failed"
    SUCCEEDED = "Succeeded"
    TIMED_OUT = "TimedOut"
    CREATED = "Created"


class ProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Provisioning state of the App Service Environment."""

    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    CANCELED = "Canceled"
    IN_PROGRESS = "InProgress"
    DELETING = "Deleting"


class RenderingType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Rendering Type."""

    NO_GRAPH = "NoGraph"
    TABLE = "Table"
    TIME_SERIES = "TimeSeries"
    TIME_SERIES_PER_INSTANCE = "TimeSeriesPerInstance"


class ResourceScopeType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Name of a resource type this recommendation applies, e.g. Subscription, ServerFarm, Site."""

    SERVER_FARM = "ServerFarm"
    SUBSCRIPTION = "Subscription"
    WEB_SITE = "WebSite"


class RouteType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of route this is:
    DEFAULT - By default, every app has routes to the local address ranges specified by RFC1918
    INHERITED - Routes inherited from the real Virtual Network routes
    STATIC - Static route set on the app only

    These values will be used for syncing an app's routes with those from a Virtual Network.
    """

    DEFAULT = "DEFAULT"
    INHERITED = "INHERITED"
    STATIC = "STATIC"


class ScmType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SCM type."""

    NONE = "None"
    DROPBOX = "Dropbox"
    TFS = "Tfs"
    LOCAL_GIT = "LocalGit"
    GIT_HUB = "GitHub"
    CODE_PLEX_GIT = "CodePlexGit"
    CODE_PLEX_HG = "CodePlexHg"
    BITBUCKET_GIT = "BitbucketGit"
    BITBUCKET_HG = "BitbucketHg"
    EXTERNAL_GIT = "ExternalGit"
    EXTERNAL_HG = "ExternalHg"
    ONE_DRIVE = "OneDrive"
    VSO = "VSO"


class SiteAvailabilityState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Management information availability state for the app."""

    NORMAL = "Normal"
    LIMITED = "Limited"
    DISASTER_RECOVERY_MODE = "DisasterRecoveryMode"


class SiteLoadBalancing(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Site load balancing."""

    WEIGHTED_ROUND_ROBIN = "WeightedRoundRobin"
    LEAST_REQUESTS = "LeastRequests"
    LEAST_RESPONSE_TIME = "LeastResponseTime"
    WEIGHTED_TOTAL_TRAFFIC = "WeightedTotalTraffic"
    REQUEST_HASH = "RequestHash"


class SkuName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SkuName."""

    FREE = "Free"
    SHARED = "Shared"
    BASIC = "Basic"
    STANDARD = "Standard"
    PREMIUM = "Premium"
    PREMIUM_V2 = "PremiumV2"
    DYNAMIC = "Dynamic"
    ISOLATED = "Isolated"


class SolutionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of Solution."""

    QUICK_SOLUTION = "QuickSolution"
    DEEP_INVESTIGATION = "DeepInvestigation"
    BEST_PRACTICES = "BestPractices"


class SslState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SSL type."""

    DISABLED = "Disabled"
    SNI_ENABLED = "SniEnabled"
    IP_BASED_ENABLED = "IpBasedEnabled"


class StatusOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """App Service plan status."""

    READY = "Ready"
    PENDING = "Pending"
    CREATING = "Creating"


class SupportedTlsVersions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """MinTlsVersion: configures the minimum version of TLS required for SSL requests."""

    ONE0 = "1.0"
    ONE1 = "1.1"
    ONE2 = "1.2"


class UsageState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """State indicating whether the app has exceeded its quota usage. Read-only."""

    NORMAL = "Normal"
    EXCEEDED = "Exceeded"


class ValidateResourceTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Resource type used for verification."""

    SERVER_FARM = "ServerFarm"
    SITE = "Site"


class WorkerSizeOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Size of the machines."""

    DEFAULT = "Default"
    SMALL = "Small"
    MEDIUM = "Medium"
    LARGE = "Large"
    D1 = "D1"
    D2 = "D2"
    D3 = "D3"



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: classification_policy_crud_ops_async.py
DESCRIPTION:
    These samples demonstrates how to create Classification Policy used in ACS JobRouter.
    You need a valid connection string to an Azure Communication Service to execute the sample

USAGE:
    python classification_policy_crud_ops_async.py
    Set the environment variables with your own values before running the sample:
    1) AZURE_COMMUNICATION_SERVICE_CONNECTION_STRING - Communication Service connection string
"""

import os
import asyncio


class ClassificationPolicySamplesAsync(object):
    connection_string = os.environ["AZURE_COMMUNICATION_SERVICE_CONNECTION_STRING"]

    _cp_policy_id = "sample_cp_policy"

    async def create_classification_policy(self):
        connection_string = self.connection_string
        policy_id = self._cp_policy_id
        # [START create_classification_policy_async]
        from azure.communication.jobrouter.aio import JobRouterAdministrationClient
        from azure.communication.jobrouter.models import (
            ClassificationPolicy,
            StaticRouterRule,
            ExpressionRouterRule,
            StaticQueueSelectorAttachment,
            ConditionalQueueSelectorAttachment,
            RouterQueueSelector,
            ConditionalWorkerSelectorAttachment,
            RouterWorkerSelector,
            LabelOperator,
        )

        # set `connection_string` to an existing ACS endpoint
        router_admin_client = JobRouterAdministrationClient.from_connection_string(conn_str=connection_string)
        print("JobRouterAdministrationClient created successfully!")

        async with router_admin_client:
            classification_policy: ClassificationPolicy = await router_admin_client.upsert_classification_policy(
                policy_id,
                ClassificationPolicy(
                    prioritization_rule=StaticRouterRule(value=10),
                    queue_selector_attachments=[
                        StaticQueueSelectorAttachment(
                            queue_selector=RouterQueueSelector(
                                key="Region", label_operator=LabelOperator.EQUAL, value="NA"
                            )
                        ),
                        ConditionalQueueSelectorAttachment(
                            condition=ExpressionRouterRule(expression='If(job.Product = "O365", true, false)'),
                            queue_selectors=[
                                RouterQueueSelector(key="Product", label_operator=LabelOperator.EQUAL, value="O365"),
                                RouterQueueSelector(key="QGroup", label_operator=LabelOperator.EQUAL, value="NA_O365"),
                            ],
                        ),
                    ],
                    worker_selector_attachments=[
                        ConditionalWorkerSelectorAttachment(
                            condition=ExpressionRouterRule(expression='If(job.Product = "O365", true, false)'),
                            worker_selectors=[
                                RouterWorkerSelector(key="Skill_O365", label_operator=LabelOperator.EQUAL, value=True),
                                RouterWorkerSelector(
                                    key="Skill_O365_Lvl", label_operator=LabelOperator.GREATER_THAN_OR_EQUAL, value=1
                                ),
                            ],
                        ),
                        ConditionalWorkerSelectorAttachment(
                            condition=ExpressionRouterRule(expression='If(job.HighPriority = "true", true, false)'),
                            worker_selectors=[
                                RouterWorkerSelector(
                                    key="Skill_O365_Lvl", label_operator=LabelOperator.GREATER_THAN_OR_EQUAL, value=10
                                )
                            ],
                        ),
                    ],
                ),
            )

            print(f"Classification Policy successfully created with id: {classification_policy.id}")

        # [END create_classification_policy_async]

    async def update_classification_policy(self):
        connection_string = self.connection_string
        policy_id = self._cp_policy_id
        # [START update_classification_policy_async]
        from azure.communication.jobrouter.aio import JobRouterAdministrationClient
        from azure.communication.jobrouter.models import (
            ClassificationPolicy,
            ExpressionRouterRule,
        )

        # set `connection_string` to an existing ACS endpoint
        router_admin_client = JobRouterAdministrationClient.from_connection_string(conn_str=connection_string)
        print("JobRouterAdministrationClient created successfully!")

        async with router_admin_client:
            updated_classification_policy: ClassificationPolicy = (
                await router_admin_client.upsert_classification_policy(
                    policy_id,
                    prioritization_rule=ExpressionRouterRule(expression='If(job.HighPriority = "true", 50, 10)'),
                )
            )

            print(f"Classification policy successfully update with new prioritization rule")
        # [END update_classification_policy_async]

    async def get_classification_policy(self):
        connection_string = self.connection_string
        policy_id = self._cp_policy_id
        # [START get_classification_policy_async]
        from azure.communication.jobrouter.aio import JobRouterAdministrationClient

        router_admin_client = JobRouterAdministrationClient.from_connection_string(conn_str=connection_string)

        async with router_admin_client:
            classification_policy = await router_admin_client.get_classification_policy(policy_id)

            print(f"Successfully fetched classification policy with id: {classification_policy.id}")
        # [END get_classification_policy_async]

    async def list_classification_policies_batched(self):
        connection_string = self.connection_string
        # [START list_classification_policies_batched_async]
        from azure.communication.jobrouter.aio import JobRouterAdministrationClient

        router_admin_client = JobRouterAdministrationClient.from_connection_string(conn_str=connection_string)

        async with router_admin_client:
            classification_policy_iterator = router_admin_client.list_classification_policies(results_per_page=10)

            async for policy_page in classification_policy_iterator.by_page():
                policies_in_page = [i async for i in policy_page]
                print(f"Retrieved {len(policies_in_page)} policies in current page")

                for cp in policies_in_page:
                    print(f"Retrieved classification policy with id: {cp.id}")

            print(f"Successfully completed fetching classification policies")
        # [END list_classification_policies_batched_async]

    async def list_classification_policies(self):
        connection_string = self.connection_string
        # [START list_classification_policies_async]
        from azure.communication.jobrouter.aio import JobRouterAdministrationClient

        router_admin_client = JobRouterAdministrationClient.from_connection_string(conn_str=connection_string)

        async with router_admin_client:
            classification_policy_iterator = router_admin_client.list_classification_policies()

            async for cp in classification_policy_iterator:
                print(f"Retrieved classification policy with id: {cp.id}")

            print(f"Successfully completed fetching classification policies")
        # [END list_classification_policies_async]

    async def clean_up(self):
        connection_string = self.connection_string
        policy_id = self._cp_policy_id

        # [START delete_classification_policy_async]
        from azure.communication.jobrouter.aio import JobRouterAdministrationClient

        router_admin_client = JobRouterAdministrationClient.from_connection_string(conn_str=connection_string)

        async with router_admin_client:
            await router_admin_client.delete_classification_policy(policy_id)

        # [END delete_classification_policy_async]


async def main():
    sample = ClassificationPolicySamplesAsync()
    await sample.create_classification_policy()
    await sample.get_classification_policy()
    await sample.update_classification_policy()
    await sample.list_classification_policies()
    await sample.list_classification_policies_batched()
    await sample.clean_up()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.dependencymap.aio import DependencyMapMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDependencyMapMgmtMapsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DependencyMapMgmtClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_get(self, resource_group):
        response = await self.client.maps.get(
            resource_group_name=resource_group.name,
            map_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.maps.begin_create_or_update(
                resource_group_name=resource_group.name,
                map_name="str",
                resource={
                    "location": "str",
                    "id": "str",
                    "name": "str",
                    "properties": {"provisioningState": "str"},
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_begin_update(self, resource_group):
        response = await (
            await self.client.maps.begin_update(
                resource_group_name=resource_group.name,
                map_name="str",
                properties={"tags": {"str": "str"}},
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_begin_delete(self, resource_group):
        response = await (
            await self.client.maps.begin_delete(
                resource_group_name=resource_group.name,
                map_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_list_by_resource_group(self, resource_group):
        response = self.client.maps.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_list_by_subscription(self, resource_group):
        response = self.client.maps.list_by_subscription()
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_begin_get_dependency_view_for_focused_machine(self, resource_group):
        response = await (
            await self.client.maps.begin_get_dependency_view_for_focused_machine(
                resource_group_name=resource_group.name,
                map_name="str",
                body={
                    "focusedMachineId": "str",
                    "filters": {
                        "dateTime": {
                            "endDateTimeUtc": "2020-02-20 00:00:00",
                            "startDateTimeUtc": "2020-02-20 00:00:00",
                        },
                        "processNameFilter": {"operator": "str", "processNames": ["str"]},
                    },
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_begin_get_connections_with_connected_machine_for_focused_machine(self, resource_group):
        response = await (
            await self.client.maps.begin_get_connections_with_connected_machine_for_focused_machine(
                resource_group_name=resource_group.name,
                map_name="str",
                body={
                    "connectedMachineId": "str",
                    "focusedMachineId": "str",
                    "filters": {
                        "dateTime": {
                            "endDateTimeUtc": "2020-02-20 00:00:00",
                            "startDateTimeUtc": "2020-02-20 00:00:00",
                        },
                        "processNameFilter": {"operator": "str", "processNames": ["str"]},
                    },
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_begin_get_connections_for_process_on_focused_machine(self, resource_group):
        response = await (
            await self.client.maps.begin_get_connections_for_process_on_focused_machine(
                resource_group_name=resource_group.name,
                map_name="str",
                body={
                    "focusedMachineId": "str",
                    "processIdOnFocusedMachine": "str",
                    "filters": {
                        "dateTime": {
                            "endDateTimeUtc": "2020-02-20 00:00:00",
                            "startDateTimeUtc": "2020-02-20 00:00:00",
                        },
                        "processNameFilter": {"operator": "str", "processNames": ["str"]},
                    },
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_maps_begin_export_dependencies(self, resource_group):
        response = await (
            await self.client.maps.begin_export_dependencies(
                resource_group_name=resource_group.name,
                map_name="str",
                body={
                    "focusedMachineId": "str",
                    "filters": {
                        "dateTime": {
                            "endDateTimeUtc": "2020-02-20 00:00:00",
                            "startDateTimeUtc": "2020-02-20 00:00:00",
                        },
                        "processNameFilter": {"operator": "str", "processNames": ["str"]},
                    },
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.devtestlabs import DevTestLabsClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDevTestLabsVirtualMachinesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DevTestLabsClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_list(self, resource_group):
        response = self.client.virtual_machines.list(
            resource_group_name=resource_group.name,
            lab_name="str",
            api_version="2018-09-15",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_get(self, resource_group):
        response = self.client.virtual_machines.get(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_create_or_update(self, resource_group):
        response = self.client.virtual_machines.begin_create_or_update(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            lab_virtual_machine={
                "allowClaim": False,
                "applicableSchedule": {
                    "id": "str",
                    "labVmsShutdown": {
                        "createdDate": "2020-02-20 00:00:00",
                        "dailyRecurrence": {"time": "str"},
                        "hourlyRecurrence": {"minute": 0},
                        "id": "str",
                        "location": "str",
                        "name": "str",
                        "notificationSettings": {
                            "emailRecipient": "str",
                            "notificationLocale": "str",
                            "status": "str",
                            "timeInMinutes": 0,
                            "webhookUrl": "str",
                        },
                        "provisioningState": "str",
                        "status": "str",
                        "tags": {"str": "str"},
                        "targetResourceId": "str",
                        "taskType": "str",
                        "timeZoneId": "str",
                        "type": "str",
                        "uniqueIdentifier": "str",
                        "weeklyRecurrence": {"time": "str", "weekdays": ["str"]},
                    },
                    "labVmsStartup": {
                        "createdDate": "2020-02-20 00:00:00",
                        "dailyRecurrence": {"time": "str"},
                        "hourlyRecurrence": {"minute": 0},
                        "id": "str",
                        "location": "str",
                        "name": "str",
                        "notificationSettings": {
                            "emailRecipient": "str",
                            "notificationLocale": "str",
                            "status": "str",
                            "timeInMinutes": 0,
                            "webhookUrl": "str",
                        },
                        "provisioningState": "str",
                        "status": "str",
                        "tags": {"str": "str"},
                        "targetResourceId": "str",
                        "taskType": "str",
                        "timeZoneId": "str",
                        "type": "str",
                        "uniqueIdentifier": "str",
                        "weeklyRecurrence": {"time": "str", "weekdays": ["str"]},
                    },
                    "location": "str",
                    "name": "str",
                    "tags": {"str": "str"},
                    "type": "str",
                },
                "artifactDeploymentStatus": {"artifactsApplied": 0, "deploymentStatus": "str", "totalArtifacts": 0},
                "artifacts": [
                    {
                        "artifactId": "str",
                        "artifactTitle": "str",
                        "deploymentStatusMessage": "str",
                        "installTime": "2020-02-20 00:00:00",
                        "parameters": [{"name": "str", "value": "str"}],
                        "status": "str",
                        "vmExtensionStatusMessage": "str",
                    }
                ],
                "computeId": "str",
                "computeVm": {
                    "dataDiskIds": ["str"],
                    "dataDisks": [{"diskSizeGiB": 0, "diskUri": "str", "managedDiskId": "str", "name": "str"}],
                    "networkInterfaceId": "str",
                    "osDiskId": "str",
                    "osType": "str",
                    "statuses": [{"code": "str", "displayStatus": "str", "message": "str"}],
                    "vmSize": "str",
                },
                "createdByUser": "str",
                "createdByUserId": "str",
                "createdDate": "2020-02-20 00:00:00",
                "customImageId": "str",
                "dataDiskParameters": [
                    {
                        "attachNewDataDiskOptions": {"diskName": "str", "diskSizeGiB": 0, "diskType": "str"},
                        "existingLabDiskId": "str",
                        "hostCaching": "str",
                    }
                ],
                "disallowPublicIpAddress": False,
                "environmentId": "str",
                "expirationDate": "2020-02-20 00:00:00",
                "fqdn": "str",
                "galleryImageReference": {
                    "offer": "str",
                    "osType": "str",
                    "publisher": "str",
                    "sku": "str",
                    "version": "str",
                },
                "id": "str",
                "isAuthenticationWithSshKey": bool,
                "labSubnetName": "str",
                "labVirtualNetworkId": "str",
                "lastKnownPowerState": "str",
                "location": "str",
                "name": "str",
                "networkInterface": {
                    "dnsName": "str",
                    "privateIpAddress": "str",
                    "publicIpAddress": "str",
                    "publicIpAddressId": "str",
                    "rdpAuthority": "str",
                    "sharedPublicIpAddressConfiguration": {
                        "inboundNatRules": [{"backendPort": 0, "frontendPort": 0, "transportProtocol": "str"}]
                    },
                    "sshAuthority": "str",
                    "subnetId": "str",
                    "virtualNetworkId": "str",
                },
                "notes": "str",
                "osType": "str",
                "ownerObjectId": "dynamicValue",
                "ownerUserPrincipalName": "str",
                "password": "str",
                "planId": "str",
                "provisioningState": "str",
                "scheduleParameters": [
                    {
                        "dailyRecurrence": {"time": "str"},
                        "hourlyRecurrence": {"minute": 0},
                        "location": "str",
                        "name": "str",
                        "notificationSettings": {
                            "emailRecipient": "str",
                            "notificationLocale": "str",
                            "status": "str",
                            "timeInMinutes": 0,
                            "webhookUrl": "str",
                        },
                        "status": "str",
                        "tags": {"str": "str"},
                        "targetResourceId": "str",
                        "taskType": "str",
                        "timeZoneId": "str",
                        "weeklyRecurrence": {"time": "str", "weekdays": ["str"]},
                    }
                ],
                "size": "str",
                "sshKey": "str",
                "storageType": "labStorageType",
                "tags": {"str": "str"},
                "type": "str",
                "uniqueIdentifier": "str",
                "userName": "str",
                "virtualMachineCreationSource": "str",
            },
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_delete(self, resource_group):
        response = self.client.virtual_machines.begin_delete(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_update(self, resource_group):
        response = self.client.virtual_machines.update(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_add_data_disk(self, resource_group):
        response = self.client.virtual_machines.begin_add_data_disk(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            data_disk_properties={
                "attachNewDataDiskOptions": {"diskName": "str", "diskSizeGiB": 0, "diskType": "str"},
                "existingLabDiskId": "str",
                "hostCaching": "str",
            },
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_apply_artifacts(self, resource_group):
        response = self.client.virtual_machines.begin_apply_artifacts(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_claim(self, resource_group):
        response = self.client.virtual_machines.begin_claim(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_detach_data_disk(self, resource_group):
        response = self.client.virtual_machines.begin_detach_data_disk(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_get_rdp_file_contents(self, resource_group):
        response = self.client.virtual_machines.get_rdp_file_contents(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_list_applicable_schedules(self, resource_group):
        response = self.client.virtual_machines.list_applicable_schedules(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_redeploy(self, resource_group):
        response = self.client.virtual_machines.begin_redeploy(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_resize(self, resource_group):
        response = self.client.virtual_machines.begin_resize(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_restart(self, resource_group):
        response = self.client.virtual_machines.begin_restart(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_start(self, resource_group):
        response = self.client.virtual_machines.begin_start(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_stop(self, resource_group):
        response = self.client.virtual_machines.begin_stop(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_transfer_disks(self, resource_group):
        response = self.client.virtual_machines.begin_transfer_disks(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_un_claim(self, resource_group):
        response = self.client.virtual_machines.begin_un_claim(
            resource_group_name=resource_group.name,
            lab_name="str",
            name="str",
            api_version="2018-09-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



from devtools_testutils import test_proxy, remove_batch_sanitizers

import pytest

# autouse=True will trigger this fixture on each pytest run, even if it's not explicitly used by a test method
@pytest.fixture(scope="session", autouse=True)
def add_sanitizers(test_proxy):
    # Remove the following sanitizers since certain fields are needed in tests and are non-sensitive:
    #  - AZSDK3433: $..userid
    remove_batch_sanitizers(["AZSDK3433"])



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.web.aio import WebSiteManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestWebSiteManagementAppServiceCertificateOrdersOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(WebSiteManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_list(self, resource_group):
        response = self.client.app_service_certificate_orders.list(
            api_version="2024-04-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_validate_purchase_information(self, resource_group):
        response = await self.client.app_service_certificate_orders.validate_purchase_information(
            app_service_certificate_order={
                "location": "str",
                "appServiceCertificateNotRenewableReasons": ["str"],
                "autoRenew": True,
                "certificates": {"str": {"keyVaultId": "str", "keyVaultSecretName": "str", "provisioningState": "str"}},
                "contact": {"email": "str", "nameFirst": "str", "nameLast": "str", "phone": "str"},
                "csr": "str",
                "distinguishedName": "str",
                "domainVerificationToken": "str",
                "expirationTime": "2020-02-20 00:00:00",
                "id": "str",
                "intermediate": {
                    "issuer": "str",
                    "notAfter": "2020-02-20 00:00:00",
                    "notBefore": "2020-02-20 00:00:00",
                    "rawData": "str",
                    "serialNumber": "str",
                    "signatureAlgorithm": "str",
                    "subject": "str",
                    "thumbprint": "str",
                    "version": 0,
                },
                "isPrivateKeyExternal": bool,
                "keySize": 2048,
                "kind": "str",
                "lastCertificateIssuanceTime": "2020-02-20 00:00:00",
                "name": "str",
                "nextAutoRenewalTimeStamp": "2020-02-20 00:00:00",
                "productType": "str",
                "provisioningState": "str",
                "root": {
                    "issuer": "str",
                    "notAfter": "2020-02-20 00:00:00",
                    "notBefore": "2020-02-20 00:00:00",
                    "rawData": "str",
                    "serialNumber": "str",
                    "signatureAlgorithm": "str",
                    "subject": "str",
                    "thumbprint": "str",
                    "version": 0,
                },
                "serialNumber": "str",
                "signedCertificate": {
                    "issuer": "str",
                    "notAfter": "2020-02-20 00:00:00",
                    "notBefore": "2020-02-20 00:00:00",
                    "rawData": "str",
                    "serialNumber": "str",
                    "signatureAlgorithm": "str",
                    "subject": "str",
                    "thumbprint": "str",
                    "version": 0,
                },
                "status": "str",
                "tags": {"str": "str"},
                "type": "str",
                "validityInYears": 1,
            },
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_list_by_resource_group(self, resource_group):
        response = self.client.app_service_certificate_orders.list_by_resource_group(
            resource_group_name=resource_group.name,
            api_version="2024-04-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_get(self, resource_group):
        response = await self.client.app_service_certificate_orders.get(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.app_service_certificate_orders.begin_create_or_update(
                resource_group_name=resource_group.name,
                certificate_order_name="str",
                certificate_distinguished_name={
                    "location": "str",
                    "appServiceCertificateNotRenewableReasons": ["str"],
                    "autoRenew": True,
                    "certificates": {
                        "str": {"keyVaultId": "str", "keyVaultSecretName": "str", "provisioningState": "str"}
                    },
                    "contact": {"email": "str", "nameFirst": "str", "nameLast": "str", "phone": "str"},
                    "csr": "str",
                    "distinguishedName": "str",
                    "domainVerificationToken": "str",
                    "expirationTime": "2020-02-20 00:00:00",
                    "id": "str",
                    "intermediate": {
                        "issuer": "str",
                        "notAfter": "2020-02-20 00:00:00",
                        "notBefore": "2020-02-20 00:00:00",
                        "rawData": "str",
                        "serialNumber": "str",
                        "signatureAlgorithm": "str",
                        "subject": "str",
                        "thumbprint": "str",
                        "version": 0,
                    },
                    "isPrivateKeyExternal": bool,
                    "keySize": 2048,
                    "kind": "str",
                    "lastCertificateIssuanceTime": "2020-02-20 00:00:00",
                    "name": "str",
                    "nextAutoRenewalTimeStamp": "2020-02-20 00:00:00",
                    "productType": "str",
                    "provisioningState": "str",
                    "root": {
                        "issuer": "str",
                        "notAfter": "2020-02-20 00:00:00",
                        "notBefore": "2020-02-20 00:00:00",
                        "rawData": "str",
                        "serialNumber": "str",
                        "signatureAlgorithm": "str",
                        "subject": "str",
                        "thumbprint": "str",
                        "version": 0,
                    },
                    "serialNumber": "str",
                    "signedCertificate": {
                        "issuer": "str",
                        "notAfter": "2020-02-20 00:00:00",
                        "notBefore": "2020-02-20 00:00:00",
                        "rawData": "str",
                        "serialNumber": "str",
                        "signatureAlgorithm": "str",
                        "subject": "str",
                        "thumbprint": "str",
                        "version": 0,
                    },
                    "status": "str",
                    "tags": {"str": "str"},
                    "type": "str",
                    "validityInYears": 1,
                },
                api_version="2024-04-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_delete(self, resource_group):
        response = await self.client.app_service_certificate_orders.delete(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_update(self, resource_group):
        response = await self.client.app_service_certificate_orders.update(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            certificate_distinguished_name={
                "appServiceCertificateNotRenewableReasons": ["str"],
                "autoRenew": True,
                "certificates": {"str": {"keyVaultId": "str", "keyVaultSecretName": "str", "provisioningState": "str"}},
                "contact": {"email": "str", "nameFirst": "str", "nameLast": "str", "phone": "str"},
                "csr": "str",
                "distinguishedName": "str",
                "domainVerificationToken": "str",
                "expirationTime": "2020-02-20 00:00:00",
                "id": "str",
                "intermediate": {
                    "issuer": "str",
                    "notAfter": "2020-02-20 00:00:00",
                    "notBefore": "2020-02-20 00:00:00",
                    "rawData": "str",
                    "serialNumber": "str",
                    "signatureAlgorithm": "str",
                    "subject": "str",
                    "thumbprint": "str",
                    "version": 0,
                },
                "isPrivateKeyExternal": bool,
                "keySize": 2048,
                "kind": "str",
                "lastCertificateIssuanceTime": "2020-02-20 00:00:00",
                "name": "str",
                "nextAutoRenewalTimeStamp": "2020-02-20 00:00:00",
                "productType": "str",
                "provisioningState": "str",
                "root": {
                    "issuer": "str",
                    "notAfter": "2020-02-20 00:00:00",
                    "notBefore": "2020-02-20 00:00:00",
                    "rawData": "str",
                    "serialNumber": "str",
                    "signatureAlgorithm": "str",
                    "subject": "str",
                    "thumbprint": "str",
                    "version": 0,
                },
                "serialNumber": "str",
                "signedCertificate": {
                    "issuer": "str",
                    "notAfter": "2020-02-20 00:00:00",
                    "notBefore": "2020-02-20 00:00:00",
                    "rawData": "str",
                    "serialNumber": "str",
                    "signatureAlgorithm": "str",
                    "subject": "str",
                    "thumbprint": "str",
                    "version": 0,
                },
                "status": "str",
                "type": "str",
                "validityInYears": 1,
            },
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_list_certificates(self, resource_group):
        response = self.client.app_service_certificate_orders.list_certificates(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            api_version="2024-04-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_get_certificate(self, resource_group):
        response = await self.client.app_service_certificate_orders.get_certificate(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_begin_create_or_update_certificate(self, resource_group):
        response = await (
            await self.client.app_service_certificate_orders.begin_create_or_update_certificate(
                resource_group_name=resource_group.name,
                certificate_order_name="str",
                name="str",
                key_vault_certificate={
                    "location": "str",
                    "id": "str",
                    "keyVaultId": "str",
                    "keyVaultSecretName": "str",
                    "kind": "str",
                    "name": "str",
                    "provisioningState": "str",
                    "tags": {"str": "str"},
                    "type": "str",
                },
                api_version="2024-04-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_delete_certificate(self, resource_group):
        response = await self.client.app_service_certificate_orders.delete_certificate(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_update_certificate(self, resource_group):
        response = await self.client.app_service_certificate_orders.update_certificate(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            name="str",
            key_vault_certificate={
                "id": "str",
                "keyVaultId": "str",
                "keyVaultSecretName": "str",
                "kind": "str",
                "name": "str",
                "provisioningState": "str",
                "type": "str",
            },
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_reissue(self, resource_group):
        response = await self.client.app_service_certificate_orders.reissue(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            reissue_certificate_order_request={
                "csr": "str",
                "delayExistingRevokeInHours": 0,
                "id": "str",
                "isPrivateKeyExternal": bool,
                "keySize": 0,
                "kind": "str",
                "name": "str",
                "type": "str",
            },
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_renew(self, resource_group):
        response = await self.client.app_service_certificate_orders.renew(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            renew_certificate_order_request={
                "csr": "str",
                "id": "str",
                "isPrivateKeyExternal": bool,
                "keySize": 0,
                "kind": "str",
                "name": "str",
                "type": "str",
            },
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_resend_email(self, resource_group):
        response = await self.client.app_service_certificate_orders.resend_email(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_resend_request_emails(self, resource_group):
        response = await self.client.app_service_certificate_orders.resend_request_emails(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            name_identifier={"name": "str"},
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_retrieve_site_seal(self, resource_group):
        response = await self.client.app_service_certificate_orders.retrieve_site_seal(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            site_seal_request={"lightTheme": bool, "locale": "str"},
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_verify_domain_ownership(self, resource_group):
        response = await self.client.app_service_certificate_orders.verify_domain_ownership(
            resource_group_name=resource_group.name,
            certificate_order_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_retrieve_certificate_actions(self, resource_group):
        response = await self.client.app_service_certificate_orders.retrieve_certificate_actions(
            resource_group_name=resource_group.name,
            name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_app_service_certificate_orders_retrieve_certificate_email_history(self, resource_group):
        response = await self.client.app_service_certificate_orders.retrieve_certificate_email_history(
            resource_group_name=resource_group.name,
            name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...
